/**
 *     Caption: Zaval Proxy Suite
 *     $Revision: 0.43 $
 *     $Date: 2002/09/26 20:20:37 $
 *
 *     @author:     Eugene Shagun
 *     @version:    1.0
 *
 * The Zaval Proxy Suite is an easy-to-use solution that allows monitoring 
 * TCP/IP-based protocols, such as HTTP, NNTP and others. It is extremely 
 * useful in software development and can be used as a debug tool. Another 
 * area of appliance is multiple connections logging with proxy facilities. 
 * As soon as it pure java solution it can be used almost everywhere.
 * 
 * For more info on this product read Zaval Proxy Suite User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/proxy/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0006/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 * 
 * Copyright (C) 2000-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
package org.zaval.io;

import java.io.*;
import java.util.*;

/**
 *  The InputIniFile class gives an access to the contents of a configuration file.
 *  The contents of ini-file is a set of pair <parameter_name> = <value>.
 *  The text after the "#" sign is meant as a comment and is not parsed.
 */

public class InputIniFile
{
  private HashMap hashObj = new HashMap();

  /**
   *  Creates an InputIniFile instance with the contents of the specified config file.
   *  @param fileName name of the configuration file to read
   *  @throws IOException if configuration file can not be read
   */
  public InputIniFile(String fileName) throws IOException
  {
    int k;
    String line;

    BufferedReader input = new BufferedReader(new FileReader(fileName));

    try
    {
      while ((line = input.readLine()) != null)
      {
        if ((k = line.indexOf('#')) != -1) line = line.substring(0, k);
        if ((k = line.indexOf('=')) == -1) continue;
        String param = line.substring(0, k).trim();
        if (param.equals("")) continue;
        String value = line.substring(k+1).trim();
        hashObj.put(param, value);
      }
    }
    finally
    {
      input.close();
    }
  }

  /**
   *  Gets the contents of the configuration file as HashMap object.
   *  @return HashMap instance containing string values of config parameters
   */
  public HashMap getTable()
  {
    return hashObj;
  }

  /**
   *  Gets a string value of the specified parameter. If this one
   *  can not be found the default value is returned.
   *  @param key parameter name
   *  @param defaultValue a default value
   *  @return a string value
   */
  public String getString(String key, String defaultValue)
  {
    String sResult = (String)hashObj.get(key);
    if (sResult == null) sResult = defaultValue;
    return sResult;
  }

  /**
   *  Gets an integer value of the specified parameter. If this one
   *  can not be found the default value is returned.
   *  @param key parameter name
   *  @param defaultValue a default value
   *  @return an integer value
   */
  public int getInt(String key, int defaultValue)
  {
    String s = (String)hashObj.get(key);
    if (s == null) return defaultValue;
    try { return Integer.parseInt(s); }
    catch (Throwable t) { return defaultValue; }
  }

  /**
   *  Gets a boolean value of the specified parameter. If this one
   *  can not be found the default value is returned.
   *  @param key parameter name
   *  @param defaultValue a default value
   *  @return a boolean value
   */
  public boolean getBoolean(String key, boolean defaultValue)
  {
    String s = (String)hashObj.get(key);
    if (s == null || s.length() == 0) return defaultValue;
    s = s.toLowerCase();
    if (trueValues.get(s) != null) return true;
    if (falseValues.get(s) != null) return false;
    return defaultValue;
  }
  
  private static HashMap trueValues;
  private static HashMap falseValues;
  
  static
  {
    trueValues = new HashMap();
    trueValues.put("on", "");
    trueValues.put("true", "");
    trueValues.put("1", "");
    
    falseValues = new HashMap();
    falseValues.put("off", "");
    falseValues.put("false", "");
    falseValues.put("0", "");
  }
}
