/**
 *     Caption: Zaval Proxy Suite
 *     $Revision: 0.43 $
 *     $Date: 2002/09/26 20:20:37 $
 *
 *     @author:     Eugene Shagun
 *     @version:    1.0
 *
 * The Zaval Proxy Suite is an easy-to-use solution that allows monitoring 
 * TCP/IP-based protocols, such as HTTP, NNTP and others. It is extremely 
 * useful in software development and can be used as a debug tool. Another 
 * area of appliance is multiple connections logging with proxy facilities. 
 * As soon as it pure java solution it can be used almost everywhere.
 * 
 * For more info on this product read Zaval Proxy Suite User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/proxy/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0006/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 * 
 * Copyright (C) 2000-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
package org.zaval.proxy;

import java.io.*;
import java.net.*;
import java.util.*;

import org.zaval.io.*;
import org.zaval.util.*;

/**
 *  The ProxyConnection object processes requests of the connected client.
 *  It redirects requests to the specified server. The responses are sent back
 *  to the client. The input and output traffic and debug information can be
 *  logged into the specified directory.
 */

public class ProxyConnection extends Thread implements Finalizable
{
  private String serverAddress;
  private int nServerPort;
  private boolean bLogging;
  private boolean bDebug;
  private String outputDir;
  private int nId;
  
  private Socket client;
  private Socket server;
  private Logger logger;
  private FileOutputStream copyInput;
  private FileOutputStream copyOutput;

  /**
   *  Creates a ProxyConnection object.
   *  @param client client socket object
   *  @param serverAddress address of the target server
   *  @param nServerPort port of the target server
   *  @param bLogging specifies if incoming traffic should be written
   *  @param bDebug specifies if debug information should be logged
   *  @param outputDir directory into which log files will be written
   *  @param nId number of the connected client, this value is used for logging
   */
  public ProxyConnection(Socket client, String serverAddress, int nServerPort, 
    boolean bLogging, boolean bDebug, String outputDir, int nId)
  {
    this.client = client;
    this.serverAddress = serverAddress;
    this.nServerPort = nServerPort;
    this.bLogging = bLogging;
    this.bDebug = bDebug;
    this.outputDir = outputDir;
    this.nId = nId;
  }

  /**
   *  Starts request processing. Connects to the target server and runs two
   *  ProxyCopier objects. One copier reads from the client input and writes into
   *  the server output. The other one reads from the server input and writes into
   *  the client output. Also log files can be opened for writing.
   */
  public void run()
  {
    try
    {
      server = new Socket(serverAddress, nServerPort);

      String id = Integer.toString(nId);
      while (id.length() < 3) id = "0" + id;

      if (bLogging)
      {
        try
        {
          copyInput = new FileOutputStream(outputDir + id + "-input");
          copyOutput = new FileOutputStream(outputDir + id + "-output");
        }
        catch (Throwable t2)
        {
          System.out.println("Error: " + t2.getMessage());
        }
      }
      
      try
      {
        if (bDebug) logger = new FileLogger(outputDir + id + "-log");
          else logger = new DummyLogger();
      }
      catch (Throwable t2)
      {
        System.out.println("Error: " + t2.getMessage());
        logger = new DummyLogger();
      }
      
      ProxyCopier copier = new RequestCopier(
        client.getInputStream(), server.getOutputStream(), copyInput, logger, this
      );
      copier.start();

      copier = new ResponseCopier(
        server.getInputStream(), client.getOutputStream(), copyOutput, logger, this
      );
      copier.start();
    }
    catch (Throwable t)
    {
      t.printStackTrace();
    }
  }

  /**
   *  Frees allocated resources. This is callback method of the Finalizable interface.
   *  It is not implemented and unused.
   */  
  public void free()
  {
  }
  
  /**
   *  Frees allocated resources. This is callback method of the Finalizable interface.
   *  Closes sockets and log files.
   *  @param obj identifying object
   */
  public void free(Object obj)
  {
    String id = (String)obj;
  
    if (id.equals("_REQ"))
    {
      try { if (copyInput != null) copyInput.close(); } catch (IOException e) {}
    }
    else
    {
      try { if (copyOutput != null) copyOutput.close(); } catch (IOException e) {}
      try { server.close(); } catch (IOException e) {}
      try { client.close(); } catch (IOException e) {}
      logger.log(">>> Sockets have been closed");
      logger.log(">>> Log is closed");
      logger.close();
    }
  }
}
