/**
 *     Caption: Zaval Proxy Suite
 *     $Revision: 0.43 $
 *     $Date: 2002/09/26 20:20:37 $
 *
 *     @author:     Eugene Shagun
 *     @version:    1.0
 *
 * The Zaval Proxy Suite is an easy-to-use solution that allows monitoring 
 * TCP/IP-based protocols, such as HTTP, NNTP and others. It is extremely 
 * useful in software development and can be used as a debug tool. Another 
 * area of appliance is multiple connections logging with proxy facilities. 
 * As soon as it pure java solution it can be used almost everywhere.
 * 
 * For more info on this product read Zaval Proxy Suite User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/proxy/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0006/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 * 
 * Copyright (C) 2000-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
package org.zaval.proxy;

import java.io.*;
import java.net.*;

import org.zaval.util.*;

/**
 *  The ProxyServer class is an implementation of proxy. The ProxyServer
 *  instance opens a server socket and listens on the specified port.
 *  All incoming requests are redirected to the target server and
 *  result is sent back to the client.
 */

public class ProxyServer extends Thread
{
  private static int SOCKET_TIMEOUT = 30000;

  private String alias;
  private int nPort;
  private String serverAddress;
  private int nServerPort;
  private boolean bLogging;
  private boolean bDebug;
  private String outputDir;
  
  private ServerSocket server;
  private NumberProvider NProvider = new NumberProvider();
  private boolean bShutdown = false;

  /**
   *  Creates a ProxyServer instance and opens a server socket to listen.
   *  @param alias display name of proxy server
   *  @param nPort listening port of proxy
   *  @param serverAddress address of the target server
   *  @param nServerPort port of the target server
   *  @param bLogging specifies if incoming traffic should be written
   *  @param bDebug specifies if debug information should be logged
   *  @param outputDir directory into which log files will be written
   *  @throws IOException if a server socket can not be created or timeout can not be set
   */
  public ProxyServer(String alias, int nPort, String serverAddress, int nServerPort,
    boolean bLogging, boolean bDebug, String outputDir) throws IOException
  {
    this.alias = alias;
    this.nPort = nPort;
    this.serverAddress = serverAddress;
    this.nServerPort = nServerPort;
    this.bLogging = bLogging;
    this.bDebug = bDebug;
    this.outputDir = outputDir;
    
    server = new ServerSocket(nPort);
    server.setSoTimeout(SOCKET_TIMEOUT);
    System.out.println("Proxy is listening on port " + nPort + " (" + alias + ")");
  }

  /**
   *  Starts a run cycle. The server socket listens and serves client requests.
   *  On each request a ProxyConnection object is created which processes this one.
   *  This cycle runs until a shutdown flag is set.
   */
  public void run()
  {
    try
    {
      for(;;)
      {
        Socket client = null;
        
        for (;;)
        {
          try
          {
            client = server.accept();
            break;
          }
          catch (InterruptedIOException e)
          {
            if (bShutdown) break;
          }
        }
        
        if (bShutdown)
        {
          if (client != null)
          {
            try { client.close(); } catch (Exception e) {}
          }
          break;
        }
        
        ProxyConnection conn = new ProxyConnection(
          client, serverAddress, nServerPort, bLogging, bDebug, outputDir, NProvider.getNumber()
        );
        conn.start();
      }
    }
    catch (Exception e)
    {}
  }

  /**
   *  Gets a listen port of the proxy.
   *  @return a port value
   */
  public int getListenPort()
  {
    return nPort;
  }

  /**
   *  Sets a shutdown flag to true.
   */  
  public void shutdown()
  {
    bShutdown = true;
  }
}